import time
import datetime as dt

import pandas as pd

from .data_reader.etc import EtcDataReader
from .data_reader.sector import SectorDataReader
from .data_reader.trader import TraderDataReader
from .data_reader.foreign import ForeignDataReader

class KrxDataReader:
    def __init__(self) -> None:
        self.etc_data_reader = EtcDataReader()
        self.sector_data_reader = SectorDataReader()
        self.base_df = self.__get_base_df()
        self.trader_data_reader = TraderDataReader(self.__scode_lcode_dict)
        self.foreign_data_reader = ForeignDataReader(self.__scode_lcode_dict)
    
    def __get_base_df(self):
        """ 기본 정보 """
        base_df = self.etc_data_reader.get_base_df()
        self.__scode_lcode_dict = base_df.set_index('단축코드')['표준코드'].to_dict()
        return base_df


from .data_reader.corp import CorpDataReader
from .data_reader.fundamental import FundamentalDataReader


class DartDataReader:
    def __init__(self, api_key):
        self.api_key = api_key
        self.corp_data_reader = CorpDataReader(api_key)
        self.fundamental_data_reader = FundamentalDataReader(api_key)
    
    



import pandas as pd
from ._data_reader import _DataReader

class TradeDataReader(_DataReader):
    """무역"""

    def get_trade_balance_df(self, start_date, end_date):
        """경상수지"""
        trade_balance_df = self.ecos_fetcher.fetch_item_value_df('301Y013', 'M', start_date, end_date)
        trade_balance_df = trade_balance_df[trade_balance_df['ITEM_CODE1'].isin(['000000', '100000', '200000', '300000', '400000'])]
        trade_balance_df = trade_balance_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        trade_balance_df.columns.name = None
        return trade_balance_df

    def get_trade_export_country_df(self, start_date, end_date):
        """수출 국가별 / 총액"""
        trade_export_df = self.ecos_fetcher.fetch_item_value_df('901Y011', 'M', start_date, end_date)
        trade_export_df = trade_export_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        trade_export_df.columns.name = None
        return trade_export_df

    def get_trade_import_country_df(self, start_date, end_date):
        """수입 국가별 / 총액"""
        trade_import_df = self.ecos_fetcher.fetch_item_value_df('901Y012', 'M', start_date, end_date)
        trade_import_df = trade_import_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        trade_import_df.columns.name = None
        return trade_import_df

    def get_trade_export_product_df(self, start_date, end_date):
        """수출 품목별 / 총액"""
        trade_export_df = self.ecos_fetcher.fetch_item_value_df('403Y001', 'M', start_date, end_date)
        code_name_dict = trade_export_df.set_index('ITEM_CODE1')['ITEM_NAME1'].to_dict()
        trade_export_df = trade_export_df.pivot(index='TIME', columns='ITEM_CODE1', values='DATA_VALUE')
        trade_export_df.columns = list(map(lambda x: code_name_dict[x], trade_export_df.columns))
        trade_export_df.columns.name = None
        return trade_export_df
    
    def get_trade_import_product_df(self, start_date, end_date):
        """수입 품목별 / 총액"""
        trade_import_df = self.ecos_fetcher.fetch_item_value_df('403Y003', 'M', start_date, end_date)
        code_name_dict = trade_import_df.set_index('ITEM_CODE1')['ITEM_NAME1'].to_dict()
        trade_import_df = trade_import_df.pivot(index='TIME', columns='ITEM_CODE1', values='DATA_VALUE')
        trade_import_df.columns = list(map(lambda x: code_name_dict[x], trade_import_df.columns))
        trade_import_df.columns.name = None
        return trade_import_df


import datetime as dt
from ..fetcher import KrxFetcher

class DataReader:
    def __init__(self):
        self.fetcher = KrxFetcher()

    @staticmethod
    def format_datetime(datetime_str):
        datetime_dt = dt.datetime.strptime(datetime_str, '%Y-%m-%d')
        return datetime_dt

    @staticmethod
    def get_latest_weekday():
        yesterday = dt.datetime.now() - dt.timedelta(days=1)
        # 주말 처리: 토요일(5)이면 금요일(-1), 일요일(6)이면 금요일(-2)
        if yesterday.weekday() == 5:  # 토요일
            latest_weekday = yesterday - dt.timedelta(days=1)
        elif yesterday.weekday() == 6:  # 일요일
            latest_weekday = yesterday - dt.timedelta(days=2)
        else:
            latest_weekday = yesterday
        return latest_weekday
    
    def split_date_ranges(self, start_date, end_date, years=2):
        start_date = self.format_datetime(start_date)
        end_date = self.format_datetime(end_date)
        ranges = []
        while start_date < end_date:
            range_end = start_date + dt.timedelta(days=years*365)
            if range_end > end_date:
                range_end = end_date
            ranges.append((start_date, range_end))
            start_date = range_end + dt.timedelta(days=1)
        return ranges


import pandas as pd
from ..fetcher import DartFetcher
from ..utils import get_chunked_list, safe_concat


class FundamentalDataReader(DartFetcher):
    def get_single_fundamental_df(self, corp_code, year, reprt):
        detail_url = 'api/fnlttSinglAcnt.json'
        resp = self.get_resp(detail_url, corp_code=corp_code, bsns_year=year, reprt_code=reprt)
        if resp:
            return self.resp2df(resp)
        else:
            return pd.DataFrame()

    def get_multi_fundamental_df(self, corp_codes, year, reprt):
        detail_url = "api/fnlttMultiAcnt.json"
        chunked_corps = get_chunked_list(corp_codes, 99)
        fundamentals_list = []
        for chunked_corp in chunked_corps:
            resp = self.get_resp(detail_url, corp_code=",".join(chunked_corp), bsns_year=year, reprt_code=reprt)
            resp_df = self.resp2df(resp)
            fundamentals_list.append(resp_df)
        return safe_concat(fundamentals_list)

    @staticmethod
    def resp2df(resp):
        try:
            resp_json = resp.json()
            resp_df = pd.DataFrame(resp_json["list"])
            return resp_df
        except:
            None



import pandas as pd
from .EcosDataReader import EcosDataReader


class MacroDataReader:
    def __init__(self, ecos_api_key):
        self.ecos_data_reader = EcosDataReader(ecos_api_key)

    def get_interest_rate_df(self, start_date, end_date):
        """
        금리 (기준금리, 예금금리, 대출금리)
        """
        df = self.ecos_data_reader.interest_data_reader.get_interest_rate_df(start_date, end_date)
        df.index.name = 'date'
        df = df.rename(columns={'기준금리': '기준금리', '수신금리': '예금금리', '대출금리': '대출금리'})

        df.index = pd.to_datetime(df.index, format='%Y%m')
        df = df.astype(float)
        return df

    def get_bond_yield_df(self, start_date, end_date):
        """
        채권 (국고채 3년 / 국고채 10년 / 국고채 20년)
        """
        df = self.ecos_data_reader.interest_data_reader.get_national_treasury_df(start_date, end_date)
        df.index.name = 'date'

        df.index = pd.to_datetime(df.index, format='%Y%m%d')
        df = df.astype(float)
        return df

    def get_exchange_rate_df(self, start_date, end_date):
        """
        환율 관련 (원/달러, 원/엔화, 원/위안, 원/유로)
        """
        df = self.ecos_data_reader.foreign_currency_data_reader.get_foreign_currency_df(start_date, end_date)
        df.index.name = 'date'

        df.index = pd.to_datetime(df.index, format='%Y%m%d')
        df = df.astype(float)
        return df

    def get_price_index_df(self, start_date, end_date):
        """
        물가 지수 (소비자 물가지수 / 생산자 물가지수 / 수출 물가지수 / 수입 물가지수)
        """
        ppi_df = self.ecos_data_reader.price_index_data_reader.get_ppi_df(start_date, end_date)
        cpi_df = self.ecos_data_reader.price_index_data_reader.get_cpi_df(start_date, end_date)
        export_pi_df = self.ecos_data_reader.price_index_data_reader.get_export_pi_df(start_date, end_date)
        import_pi_df = self.ecos_data_reader.price_index_data_reader.get_import_pi_df(start_date, end_date)
        df = pd.concat(
            [
                cpi_df.loc[:, ['총지수']].rename(columns={'총지수': '소비자물가지수'}),
                ppi_df.loc[:, ['총지수']].rename(columns={'총지수': '생산자물가지수'}),
                export_pi_df.loc[:, ['총지수']].rename(columns={'총지수': '수출물가지수'}),
                import_pi_df.loc[:, ['총지수']].rename(columns={'총지수': '수입물가지수'}),
            ],
            axis=1,
        )
        df.index.name = 'date'
        df.index = pd.to_datetime(df.index, format='%Y%m')
        df = df.astype(float)
        return df

    def get_trade_balance_df(self, start_date, end_date):
        """
        무역 (무역수지 / 수출 총지수 / 수출 총액 / 수입 총지수 / 수입 총액)
        """
        trade_balance_df = self.ecos_data_reader.trade_data_reader.get_trade_balance_df(start_date, end_date)
        export_country_df = self.ecos_data_reader.trade_data_reader.get_trade_export_country_df(start_date, end_date)
        export_product_df = self.ecos_data_reader.trade_data_reader.get_trade_export_product_df(start_date, end_date)
        import_country_df = self.ecos_data_reader.trade_data_reader.get_trade_import_country_df(start_date, end_date)
        import_product_df = self.ecos_data_reader.trade_data_reader.get_trade_import_product_df(start_date, end_date)
        df = pd.concat(
            [
                trade_balance_df.loc[:, ['경상수지']].rename(columns={'경상수지': '무역수지'}),
                export_product_df.loc[:, ['총지수']].rename(columns={'총지수': '수출_총지수'}),
                export_country_df.loc[:, ['국별수출(관세청)']].rename(columns={'국별수출(관세청)': '수출_총액'}),
                import_product_df.loc[:, ['총지수']].rename(columns={'총지수': '수입_총지수'}),
                import_country_df.loc[:, ['국별수입(관세청)']].rename(columns={'국별수입(관세청)': '수입_총액'}),
            ],
            axis=1,
        )
        df.index.name = 'date'
        df.index = pd.to_datetime(df.index, format='%Y%m')
        df = df.astype(float)
        return df



from .data_reader.economic_index import EconomicIndexDataReader
from .data_reader.price_index import PriceIndexDataReader
from .data_reader.interest import InterestDataReader

from .data_reader.global_index import GlobalIndexDataReader
from .data_reader.foreign_currency import ForeignCurrencyDataReader

from .data_reader.bank import BankDataReader
from .data_reader.monetary import MonetaryDataReader

from .data_reader.market import MarketDataReader

from .data_reader.payment import PaymentDataReader
from .data_reader.trade import TradeDataReader


class EcosDataReader:
    def __init__(self, api_key):
        self.api_key = api_key
        self.economic_index_data_reader = EconomicIndexDataReader(api_key)
        self.price_index_data_reader = PriceIndexDataReader(api_key)
        self.interest_data_reader = InterestDataReader(api_key)
        self.global_index_data_reader = GlobalIndexDataReader(api_key)
        self.foreign_currency_data_reader = ForeignCurrencyDataReader(api_key)
        self.bank_data_reader = BankDataReader(api_key)
        self.monetary_data_reader = MonetaryDataReader(api_key)
        self.market_data_reader = MarketDataReader(api_key)
        self.payment_data_reader = PaymentDataReader(api_key)
        self.trade_data_reader = TradeDataReader(api_key)



import pandas as pd
import zipfile
import xmltodict
from io import BytesIO
from ..fetcher import DartFetcher

class CorpDataReader(DartFetcher):
    def get_corp_df(self):
        resp = self.get_resp('api/corpCode.xml')
        if resp:
            resp_dict = self.resp2dict(resp)
            resp_df = pd.DataFrame(resp_dict['result']['list'])
            return resp_df
        return None

    @staticmethod
    def resp2dict(resp) -> dict:
        """
        input : bytes (zip file -> that contains COPRCODE.xml)
        return : dict
        """
        with zipfile.ZipFile(BytesIO(resp.content)) as zip_r:
            with zip_r.open('CORPCODE.xml') as f:
                resp_dict = xmltodict.parse(f.read())
        return resp_dict


import pandas as pd
from ._data_reader import _DataReader


class MarketDataReader(_DataReader):

    def get_kospi_df(self, start_date, end_date):
        """KOSPI 지수, 거래량, 거래대금, 외국인순매수"""
        # kospi_지수
        kospi_value = self.ecos_fetcher.fetch_item_value_df('802Y001', 'D', start_date, end_date, '0001000')
        kospi_value = self.df2series(kospi_value, '코스피_지수')
        # kospi_거래량
        kospi_volume_cnt = self.ecos_fetcher.fetch_item_value_df('802Y001', 'D', start_date, end_date, '0002000')
        kospi_volume_cnt = self.df2series(kospi_volume_cnt, '코스피_거래량')
        # kospi_거래대금
        kospi_volume_won = self.ecos_fetcher.fetch_item_value_df('802Y001', 'D', start_date, end_date, '0003000')
        kospi_volume_won = self.df2series(kospi_volume_won, '코스피_거래대금')
        # kospi_외국인순매수
        kospi_foreign_buy = self.ecos_fetcher.fetch_item_value_df('802Y001', 'D', start_date, end_date, '0030000')
        kospi_foreign_buy = self.df2series(kospi_foreign_buy, '코스피_외국인순매수')
        kospi_df = pd.concat([kospi_value, kospi_volume_cnt, kospi_volume_won, kospi_foreign_buy], axis=1)
        return kospi_df

    def get_kosdaq_df(self, start_date, end_date):
        """KOSDAQ 지수, 거래량, 거래대금, 외국인순매수"""
        # kosdaq_지수
        kosdaq_value = self.ecos_fetcher.fetch_item_value_df('802Y001', 'D', start_date, end_date, '0089000')
        kosdaq_value = self.df2series(kosdaq_value, '코스닥_지수')
        # kosdaq_거래량
        kosdaq_volume_cnt = self.ecos_fetcher.fetch_item_value_df('802Y001', 'D', start_date, end_date, '0090000')
        kosdaq_volume_cnt = self.df2series(kosdaq_volume_cnt, '코스닥_거래량')
        # kosdaq_거래대금
        kosdaq_volume_won = self.ecos_fetcher.fetch_item_value_df('802Y001', 'D', start_date, end_date, '0091000')
        kosdaq_volume_won = self.df2series(kosdaq_volume_won, '코스닥_거래대금')
        # kosdaq_외국인순매수
        kosdaq_foreign_buy = self.ecos_fetcher.fetch_item_value_df('802Y001', 'D', start_date, end_date, '0113000')
        kosdaq_foreign_buy = self.df2series(kosdaq_foreign_buy, '코스닥_외국인순매수')
        kosdaq_df = pd.concat([kosdaq_value, kosdaq_volume_cnt, kosdaq_volume_won, kosdaq_foreign_buy], axis=1)
        return kosdaq_df

    def get_balance_df(self, start_date, end_date):
        """예탁금 / 파생거래 예탁금 / 미수금"""
        # 투자자 예탁금
        invest_deposit = self.ecos_fetcher.fetch_item_value_df('901Y056', 'M', start_date, end_date, 'S23A')
        invest_deposit = self.df2series(invest_deposit, '투자자_예탁금')

        # 파생거래 예탁금
        derivative_deposit = self.ecos_fetcher.fetch_item_value_df('901Y056', 'M', start_date, end_date, 'S23B')
        derivative_deposit = self.df2series(derivative_deposit, '파생거래_예탁금')

        # 미수금
        unpaid_balance = self.ecos_fetcher.fetch_item_value_df('901Y056', 'M', start_date, end_date, 'S23D')
        unpaid_balance = self.df2series(unpaid_balance, '미수금')

        balance_df = pd.concat([invest_deposit, derivative_deposit, unpaid_balance], axis=1)
        return balance_df

    def get_derivative_df(self, start_date, end_date):
        """파생거래 관련 데이터"""
        # 선물_계약수
        future_volume_cnt = self.ecos_fetcher.fetch_item_value_df('901Y057', 'M', start_date, end_date, 'S25A')
        future_volume_cnt = self.df2series(future_volume_cnt, '선물_계약수')

        # 선물_계약금액
        future_volume_won = self.ecos_fetcher.fetch_item_value_df('901Y057', 'M', start_date, end_date, 'S25C')
        future_volume_won = self.df2series(future_volume_won, '선물_계약금액')

        # CALL_옵션_계약수
        call_option_volume_cnt = self.ecos_fetcher.fetch_item_value_df('901Y058', 'M', start_date, end_date, 'S26BA')
        call_option_volume_cnt = self.df2series(call_option_volume_cnt, 'CALL_옵션_계약수')

        # CALL_옵션_계약금액
        call_option_volume_won = self.ecos_fetcher.fetch_item_value_df('901Y058', 'M', start_date, end_date, 'S26BC')
        call_option_volume_won = self.df2series(call_option_volume_won, 'CALL_옵션_계약금액')

        # PUT_옵션_계약수
        put_option_volume_cnt = self.ecos_fetcher.fetch_item_value_df('901Y058', 'M', start_date, end_date, 'S26CA')
        put_option_volume_cnt = self.df2series(put_option_volume_cnt, 'PUT_옵션_계약수')

        # PUT_옵션_계약금액
        put_option_volume_won = self.ecos_fetcher.fetch_item_value_df('901Y058', 'M', start_date, end_date, 'S26CC')
        put_option_volume_won = self.df2series(put_option_volume_won, 'PUT_옵션_계약금액')

        derivative_df = pd.concat(
            [
                future_volume_cnt,
                future_volume_won,
                call_option_volume_cnt,
                call_option_volume_won,
                put_option_volume_cnt,
                put_option_volume_won,
            ],
            axis=1,
        )
        return derivative_df



import pandas as pd
from ._data_reader import _DataReader


class GlobalIndexDataReader(_DataReader):
    def get_global_interest_df(self, start_date, end_date):
        """주요국 기준금리"""
        global_interest_df = self.ecos_fetcher.fetch_item_value_df('902Y006', 'M', start_date, end_date)
        global_interest_df = global_interest_df[global_interest_df['ITEM_CODE1'].isin(['KR', 'US', 'XM', 'JP'])]
        global_interest_df = global_interest_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        global_interest_df.columns.name = None
        return global_interest_df

    def get_global_short_interest_df(self, start_date, end_date):
        """주요국 단기금리"""
        global_short_interest_df = self.ecos_fetcher.fetch_item_value_df('902Y023', 'M', start_date, end_date)
        global_short_interest_df = global_short_interest_df[global_short_interest_df['ITEM_CODE1'] == 'IR3TIB']
        global_short_interest_df = global_short_interest_df.pivot(index='TIME', columns='ITEM_NAME2', values='DATA_VALUE')
        global_short_interest_df.columns.name = None
        return global_short_interest_df

    def get_global_long_interest_df(self, start_date, end_date):
        """주요국 장기금리"""
        global_long_interest_df = self.ecos_fetcher.fetch_item_value_df('902Y023', 'M', start_date, end_date)
        global_long_interest_df = global_long_interest_df[global_long_interest_df['ITEM_CODE1'] == 'IRLT']
        global_long_interest_df = global_long_interest_df.pivot(index='TIME', columns='ITEM_NAME2', values='DATA_VALUE')
        global_long_interest_df.columns.name = None
        return global_long_interest_df

    def get_global_market_index_df(self, start_date, end_date):
        """주요국 주가지수"""
        global_market_index_df = self.ecos_fetcher.fetch_item_value_df('902Y002', 'M', start_date, end_date)
        global_market_index_df = global_market_index_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        global_market_index_df.columns.name = None
        return global_market_index_df

    def get_global_ppi_df(self, start_date, end_date):
        """주요국 생산자물가지수"""
        global_ppi_df = self.ecos_fetcher.fetch_item_value_df('902Y007', 'M', start_date, end_date)
        global_ppi_df = global_ppi_df[global_ppi_df['ITEM_CODE1'].isin(['KR', 'US', 'CN', 'JP'])]
        global_ppi_df = global_ppi_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        global_ppi_df.columns.name = None
        return global_ppi_df

    def get_global_cpi_df(self, start_date, end_date):
        """주요국 소비자물가지수"""
        global_cpi_df = self.ecos_fetcher.fetch_item_value_df('902Y008', 'M', start_date, end_date)
        global_cpi_df = global_cpi_df[global_cpi_df['ITEM_CODE1'].isin(['KR', 'US', 'CN', 'JP'])]
        global_cpi_df = global_cpi_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        global_cpi_df.columns.name = None
        return global_cpi_df



from .base import DataReader

class EtcDataReader(DataReader):
    def __init__(self):
        super().__init__()
        
    def get_base_df(self):
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01901",
            "locale": "ko_KR",
            "mktId": "ALL",
            "share": "1",
            "csvxls_isNo": "false",
        }
        df = self.fetcher.fetch(data, 'OutBlock_1')
        df.columns = [
            '표준코드','단축코드','한글종목명','한글종목약명','영문종목명','상장일',
            '시장구분','증권구분','소속부','주식종류','액면가','상장주식수'
            ]
        return df

    def get_dividend_df(self):
        """배당 정보 추출"""
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT03501',
            'locale': 'ko_KR',
            'searchType': '1',
            'mktId': 'ALL',
            'trdDd': self.get_latest_weekday().strftime("%Y%m%d"),
            'csvxls_isNo': 'false',
        }
        df = self.fetcher.fetch(data, 'output')
        df = df.loc[:,['ISU_SRT_CD','ISU_ABBRV','TDD_CLSPRC','DPS']]
        df.columns = [
            '종목코드','종목명','종가','주당배당금'
            ]
        return df



from .DartDataReader import DartDataReader


class CorpDataReader:
    def __init__(self, dart_api_key):
        self.dart_data_reader = DartDataReader(dart_api_key)
        self.corp_df = self.dart_data_reader.corp_data_reader.get_corp_df()
        self.stock_corp_dict = self.corp_df.set_index('stock_code')['corp_code'].to_dict()

    def get_fundamental_df(self, stock_code, reprt_code, year):
        corp_code = self.stock_corp_dict[stock_code]
        fundamental_df = self.dart_data_reader.fundamental_data_reader.get_single_fundamental_df(corp_code, year, reprt_code)
        return fundamental_df

    def get_fundamentals_df(self, stock_codes, reprt_code, year):
        corp_codes = list(self.corp_df[self.corp_df['stock_code'].isin(stock_codes)]['corp_code'])
        fundamental_df = self.dart_data_reader.fundamental_data_reader.get_multi_fundamental_df(corp_codes, year, reprt_code)
        return fundamental_df



import requests
import datetime as dt
import pandas as pd


class KrxFetcher:
    def __init__(self) -> None:
        self.headers = {
            'User-Agent': 'Chrome/78.0.3904.87 Safari/537.36',
            'Referer': 'http://data.krx.co.kr/',
        }
        self.url = 'http://data.krx.co.kr/comm/bldAttendant/getJsonData.cmd'

    def fetch(self, data, key):
        return self.data2df(data, key)

    def fetch_with_date_ranges(self, data, key, date_ranges):
        df_list = []
        for start_date, end_date in date_ranges:
            data['strtDd'] = start_date.strftime("%Y%m%d")
            data['endDd'] = end_date.strftime("%Y%m%d")
            _df = self.data2df(data, key)
            df_list.append(_df)
        return pd.concat(df_list, axis=0)

    def data2df(self, data, key):
        resp = requests.post(self.url, data, headers=self.headers)
        df = pd.DataFrame(resp.json().get(key))
        return df



import pandas as pd
from ._data_reader import _DataReader


class ForeignCurrencyDataReader(_DataReader):

    def get_foreign_currency_df(self, start_date, end_date):
        won_dollar = self.ecos_fetcher.fetch_item_value_df('731Y001', 'D', start_date, end_date, '0000001')
        won_dollar = self.df2series(won_dollar, '원_달러')

        won_yen = self.ecos_fetcher.fetch_item_value_df('731Y001', 'D', start_date, end_date, '0000002')
        won_yen = self.df2series(won_yen, '원_엔')

        won_euro = self.ecos_fetcher.fetch_item_value_df('731Y001', 'D', start_date, end_date, '0000003')
        won_euro = self.df2series(won_euro, '원_유로')

        won_yuan = self.ecos_fetcher.fetch_item_value_df('731Y001', 'D', start_date, end_date, '0000053')
        won_yuan = self.df2series(won_yuan, '원_위안')

        foreign_currency_df = pd.concat([won_dollar, won_yen, won_euro, won_yuan], axis=1)
        return foreign_currency_df



import urllib
import requests

class DartFetcher:
    def __init__(self, api_key) -> None:
        self.api_key = api_key
        self.base_url = 'https://opendart.fss.or.kr'
        self.base_params = {'crtfc_key': api_key}

    def format_url(self, detail_url):
        url = urllib.parse.urljoin(self.base_url, detail_url)
        return url

    def format_params(self, **detail_params):
        params = {**self.base_params, **detail_params}
        return params

    def get_request_params(self, detail_url, **detail_params):
        request_params = {
            "url": self.format_url(detail_url),
            "params": self.format_params(**detail_params),
        }
        return request_params

    def get_resp(self, detail_url, **detail_params):
        try:
            resp = requests.get(**self.get_request_params(detail_url, **detail_params))
            return resp
        except requests.exceptions.SSLError:
            print('SSL error occurred: might be a problem with the [IP or SSL certification.]')
            return None

        except requests.exceptions.HTTPError:
            print(
                'HTTP error occurred: might be a problem with the [internet connection or the server].'
            )
            return None



import pandas as pd
from ._data_reader import _DataReader


class PaymentDataReader(_DataReader):
    def get_payment_cnt_df(self, start_date, end_date):
        """지급수단 건별"""
        payment_cnt_df = self.ecos_fetcher.fetch_item_value_df('602Y002', 'M', start_date, end_date)
        payment_cnt_df = payment_cnt_df[
            (payment_cnt_df['ITEM_CODE1'].isin(['D0000', 'D1000', 'D2000', 'E0000', 'E1000', 'E1100', 'E1300', 'E3000']))
            & (payment_cnt_df['ITEM_CODE2'] == 'EACH')
        ]
        payment_cnt_df = payment_cnt_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        payment_cnt_df.columns.name = None
        return payment_cnt_df

    def get_payment_won_df(self, start_date, end_date):
        """지급수단 총액"""
        payment_won_df = self.ecos_fetcher.fetch_item_value_df('602Y002', 'M', start_date, end_date)
        payment_won_df = payment_won_df[
            (payment_won_df['ITEM_CODE1'].isin(['D0000', 'D1000', 'D2000', 'E0000', 'E1000', 'E1100', 'E1300', 'E3000']))
            & (payment_won_df['ITEM_CODE2'] == 'MONEY')
        ]
        payment_won_df = payment_won_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        payment_won_df.columns.name = None
        return payment_won_df

    def get_total_credit_card_df(self, start_date, end_date):
        """개인 총합 신용카드"""
        total_credit_card_df = self.ecos_fetcher.fetch_item_value_df('601Y003', 'M', start_date, end_date)
        total_credit_card_df = total_credit_card_df[
            total_credit_card_df['ITEM_CODE1'].isin(
                ['101000', '101010', '101020', '101030', '201000', '201010', '201020', '201030', '301000']
            )
        ]
        total_credit_card_df = total_credit_card_df[total_credit_card_df['ITEM_NAME2'] == '합계'].pivot(
            index='TIME', columns='ITEM_NAME1', values='DATA_VALUE'
        )
        total_credit_card_df.columns.name = None
        return total_credit_card_df

    def get_bank_credit_card_df(self, start_date, end_date):
        """개인 은행권 신용카드"""
        bank_credit_card_df = self.ecos_fetcher.fetch_item_value_df('601Y003', 'M', start_date, end_date)
        bank_credit_card_df = bank_credit_card_df[
            bank_credit_card_df['ITEM_CODE1'].isin(
                ['101000', '101010', '101020', '101030', '201000', '201010', '201020', '201030', '301000']
            )
        ]
        bank_credit_card_df = bank_credit_card_df[bank_credit_card_df['ITEM_NAME2'] == '은행계'].pivot(
            index='TIME', columns='ITEM_NAME1', values='DATA_VALUE'
        )
        bank_credit_card_df.columns.name = None
        return bank_credit_card_df

    def get_not_bank_credit_card_df(self, start_date, end_date):
        """개인 비은행권 신용카드"""
        not_bank_credit_card_df = self.ecos_fetcher.fetch_item_value_df('601Y003', 'M', start_date, end_date)
        not_bank_credit_card_df = not_bank_credit_card_df[
            not_bank_credit_card_df['ITEM_CODE1'].isin(
                ['101000', '101010', '101020', '101030', '201000', '201010', '201020', '201030', '301000']
            )
        ]
        not_bank_credit_card_df = not_bank_credit_card_df[not_bank_credit_card_df['ITEM_NAME2'] == '비은행계'].pivot(
            index='TIME', columns='ITEM_NAME1', values='DATA_VALUE'
        )
        not_bank_credit_card_df.columns.name = None
        return not_bank_credit_card_df



import requests
import pandas as pd

import datetime as dt


class EcosFetcher:
    def __init__(self, api_key):
        self.api_key = api_key
        self.base_url = 'https://ecos.bok.or.kr/api'

    @staticmethod
    def _format_time(period, datetime_str):
        datetime_dt = dt.datetime.strptime(datetime_str, "%Y-%m-%d")
        if period == 'A':
            return datetime_dt.strftime("%Y")
        elif period == 'Q':
            quarter = (datetime_dt.month - 1) // 3 + 1
            return f"{datetime_dt.year}Q{quarter}"
        elif period == 'M':
            return datetime_dt.strftime("%Y%m")
        elif period == 'D':
            return datetime_dt.strftime("%Y%m%d")
        else:
            raise ValueError('Invalid Time')

    def _get_resp(self, params):
        query_params = '/'.join(params.values())
        url = f"{self.base_url}/{query_params}"
        resp = requests.get(url)
        return resp

    def fetch_stat_code_df(self):
        params = {
            "서비스명": "StatisticTableList",
            "인증키": self.api_key,
            "요청유형": "json",
            "언어구분": "kr",
            "요청시작건수": "1",
            "요청종료건수": "99999",
            # "통계표코드": 통계표코드,
        }
        resp = self._get_resp(params)
        stat_code_df = pd.DataFrame(resp.json()['StatisticTableList']['row'])
        return stat_code_df

    def fetch_item_code_df(self, stat_code):
        params = {
            "서비스명": "StatisticItemList",
            "인증키": self.api_key,
            "요청유형": "json",
            "언어구분": "kr",
            "요청시작건수": "1",
            "요청종료건수": "99999",
            "통계표코드": stat_code,
        }
        resp = self._get_resp(params)
        item_code_df = pd.DataFrame(resp.json()['StatisticItemList']['row'])
        return item_code_df

    def fetch_item_value_df(
        self,
        stat_code,
        period,
        start_date,
        end_date,
        통계항목코드1=None,
        통계항목코드2=None,
        통계항목코드3=None,
        통계항목코드4=None,
    ):
        params = {
            "서비스명": "StatisticSearch",
            "인증키": self.api_key,
            "요청유형": "json",
            "언어구분": "kr",
            "요청시작건수": "1",
            "요청종료건수": "99999",
            "통계표코드": stat_code,
            "주기": period,
            "검색시작일자": self._format_time(period, start_date),
            "검색종료일자": self._format_time(period, end_date),
            "통계항목코드1": '' if 통계항목코드1 is None else 통계항목코드1,
            "통계항목코드2": '' if 통계항목코드2 is None else 통계항목코드2,
            "통계항목코드3": '' if 통계항목코드3 is None else 통계항목코드3,
            "통계항목코드4": '' if 통계항목코드4 is None else 통계항목코드4,
        }
        resp = self._get_resp(params)
        item_code_df = pd.DataFrame(resp.json()['StatisticSearch']['row'])
        return item_code_df



import pandas as pd
from ._data_reader import _DataReader


class MonetaryDataReader(_DataReader):
    def get_m0_df(self, start_date, end_date):
        """본원 통화"""
        m0_last_raw = self.ecos_fetcher.fetch_item_value_df('102Y001', 'M', start_date, end_date, 'ABA2')
        m0_last_raw = self.df2series(m0_last_raw, '본원_말잔_원계열')

        m0_avg_raw = self.ecos_fetcher.fetch_item_value_df('102Y002', 'M', start_date, end_date, 'ABA1')
        m0_avg_raw = self.df2series(m0_avg_raw, '본원_평잔_원계열')

        m0_last_season = self.ecos_fetcher.fetch_item_value_df('102Y003', 'M', start_date, end_date, 'ABA2')
        m0_last_season = self.df2series(m0_last_season, '본원_말잔_계절조정')

        m0_avg_season = self.ecos_fetcher.fetch_item_value_df('102Y004', 'M', start_date, end_date, 'ABA1')
        m0_avg_season = self.df2series(m0_avg_season, '본원_평잔_계절조정')

        m0_df = pd.concat([m0_avg_raw, m0_avg_season, m0_last_raw, m0_last_season], axis=1)
        return m0_df

    def get_m1_df(self, start_date, end_date):
        """협의 통화"""
        m1_last_season = self.ecos_fetcher.fetch_item_value_df('101Y016', 'M', start_date, end_date, 'BBKS00')
        m1_last_season = self.df2series(m1_last_season, 'M1_말잔_계절조정')

        m1_last_raw = self.ecos_fetcher.fetch_item_value_df('101Y017', 'M', start_date, end_date, 'BBKA00')
        m1_last_raw = self.df2series(m1_last_raw, 'M1_말잔_원계열')

        m1_avg_season = self.ecos_fetcher.fetch_item_value_df('101Y018', 'M', start_date, end_date, 'BBLS00')
        m1_avg_season = self.df2series(m1_avg_season, 'M1_평잔_계절조정')

        m1_avg_raw = self.ecos_fetcher.fetch_item_value_df('101Y019', 'M', start_date, end_date, 'BBLA00')
        m1_avg_raw = self.df2series(m1_avg_raw, 'M1_평잔_원계열')

        m1_df = pd.concat([m1_avg_raw, m1_avg_season, m1_last_raw, m1_last_season], axis=1)
        return m1_df

    def get_m2_df(self, start_date, end_date):
        """광의 통화"""
        m2_last_season = self.ecos_fetcher.fetch_item_value_df('101Y001', 'M', start_date, end_date, 'BBGS00')
        m2_last_season = self.df2series(m2_last_season, 'M2_말잔_계절조정')

        m2_last_raw = self.ecos_fetcher.fetch_item_value_df('101Y002', 'M', start_date, end_date, 'BBGA00')
        m2_last_raw = self.df2series(m2_last_raw, 'M2_말잔_원계열')

        m2_avg_season = self.ecos_fetcher.fetch_item_value_df('101Y003', 'M', start_date, end_date, 'BBHS00')
        m2_avg_season = self.df2series(m2_avg_season, 'M2_평잔_계절조정')

        m2_avg_raw = self.ecos_fetcher.fetch_item_value_df('101Y004', 'M', start_date, end_date, 'BBHA00')
        m2_avg_raw = self.df2series(m2_avg_raw, 'M2_평잔_원계열')

        m2_df = pd.concat([m2_avg_raw, m2_avg_season, m2_last_raw, m2_last_season], axis=1)
        return m2_df

    def get_m2_ratio_df(self, start_date, end_date):
        """광의통화 말잔-원계열-비율"""
        m2_household = self.ecos_fetcher.fetch_item_value_df('101Y014', 'M', start_date, end_date, 'BBGAJ1')
        m2_household = self.df2series(m2_household, 'M2_가계_비영리단체')

        m2_corporate = self.ecos_fetcher.fetch_item_value_df('101Y014', 'M', start_date, end_date, 'BBGAJ2')
        m2_corporate = self.df2series(m2_corporate, 'M2_기업')

        m2_financial_corporate = self.ecos_fetcher.fetch_item_value_df('101Y014', 'M', start_date, end_date, 'BBGAJ3')
        m2_financial_corporate = self.df2series(m2_financial_corporate, 'M2_기타_금융기관')

        m2_etc = self.ecos_fetcher.fetch_item_value_df('101Y014', 'M', start_date, end_date, 'BBGAJ4')
        m2_etc = self.df2series(m2_etc, 'M2_기타')

        m2_last_raw_ratio_df = pd.concat([m2_household, m2_corporate, m2_financial_corporate, m2_etc], axis=1)
        return m2_last_raw_ratio_df



import pandas as pd
from ._data_reader import _DataReader


class PriceIndexDataReader(_DataReader):
    def get_ppi_df(self, start_date, end_date):
        """생산자물가지수"""
        ppi_df = self.ecos_fetcher.fetch_item_value_df('404Y014', 'M', start_date, end_date)
        ppi_df = ppi_df[ppi_df['ITEM_CODE1'].isin(['*AA', '1AA', '2AA', '3AA', '4AA', '5AA'])]
        ppi_df = ppi_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        ppi_df.columns.name = None
        return ppi_df

    def get_cpi_df(self, start_date, end_date):
        """소비자물가지수"""
        cpi_df = self.ecos_fetcher.fetch_item_value_df('901Y009', 'M', start_date, end_date)
        cpi_df = cpi_df[cpi_df['ITEM_CODE1'].isin(['0', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L'])]
        cpi_df = cpi_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        cpi_df.columns.name = None
        return cpi_df

    def get_export_pi_df(self, start_date, end_date):
        """수출물가지수"""
        export_pi_df = self.ecos_fetcher.fetch_item_value_df('402Y014', 'M', start_date, end_date)
        export_pi_df = export_pi_df[
            (export_pi_df['ITEM_CODE1'].isin(['*AA', '1AA', '3AA'])) & (export_pi_df['ITEM_CODE2'] == 'D')
        ]
        export_pi_df = export_pi_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        export_pi_df.columns.name = None
        return export_pi_df

    def get_import_pi_df(self, start_date, end_date):
        """수입물가지수"""
        import_pi_df = self.ecos_fetcher.fetch_item_value_df('401Y015', 'M', start_date, end_date)
        import_pi_df = import_pi_df[
            (import_pi_df['ITEM_CODE1'].isin(['*AA', '1AA', '2AA', '3AA'])) & (import_pi_df['ITEM_CODE2'] == 'D')
        ]
        import_pi_df = import_pi_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        import_pi_df.columns.name = None
        return import_pi_df

    def get_housing_price_df(self, start_date, end_date):
        """주택매매가격지수"""
        housing_price_df = self.ecos_fetcher.fetch_item_value_df('901Y062', 'M', start_date, end_date)
        housing_price_df = housing_price_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        housing_price_df.columns.name = None
        return housing_price_df



from .base import DataReader


class TraderDataReader(DataReader):
    def __init__(self, scode_lcode_dict):
        super().__init__()
        self.scode_lcode_dict = scode_lcode_dict

    def get_kospi_buy_trade_df(self, start_date, end_date):
        """코스피 매수 거래자 추출"""
        date_ranges = self.split_date_ranges(start_date, end_date, 2)
        data = self.get_market_trader_data()
        data['askBid'] = '2'
        data['mktId'] = 'STK'
        data['etf'] = 'EF'
        data['etn'] = 'EN'
        data['elw'] = 'EW'
        df = self.fetcher.fetch_with_date_ranges(data,'output', date_ranges)
        df.columns = [
            '날짜', '금융투자', '보험', '투신', '사모', '은행','기타금융', '연기금', '기타법인',
            '개인', '외국인','기타외국인', '전체'
            ]
        return df

    def get_kospi_sell_trade_df(self, start_date, end_date):
        """코스피 매도 거래자 추출"""
        date_ranges = self.split_date_ranges(start_date, end_date, 2)
        data = self.get_market_trader_data()
        data['askBid'] = '1'
        data['mktId'] = 'STK'
        data['etf'] = 'EF'
        data['etn'] = 'EN'
        data['elw'] = 'EW'
        df = self.fetcher.fetch_with_date_ranges(data,'output', date_ranges)
        df.columns = [
            '날짜', '금융투자', '보험', '투신', '사모', '은행','기타금융', '연기금', '기타법인',
            '개인', '외국인','기타외국인', '전체'
            ]
        return df

    def get_kosdaq_buy_trade_df(self, start_date, end_date):
        """코스닥 매수 거래자 추출"""
        date_ranges = self.split_date_ranges(start_date, end_date, 2)
        data = self.get_market_trader_data()
        data['askBid'] = '2'
        data['mktId'] = 'KSQ'
        df = self.fetcher.fetch_with_date_ranges(data,'output', date_ranges)
        df.columns = [
            '날짜', '금융투자', '보험', '투신', '사모', '은행','기타금융', '연기금', '기타법인',
            '개인', '외국인','기타외국인', '전체'
            ]
        return df

    def get_kosdaq_sell_trade_df(self, start_date, end_date):
        """코스닥 매도 거래자 추출"""
        date_ranges = self.split_date_ranges(start_date, end_date, 2)
        data = self.get_market_trader_data()
        data['askBid'] = '1'
        data['mktId'] = 'KSQ'
        df = self.fetcher.fetch_with_date_ranges(data,'output', date_ranges)
        df.columns = [
            '날짜', '금융투자', '보험', '투신', '사모', '은행','기타금융', '연기금', '기타법인',
            '개인', '외국인','기타외국인', '전체'
            ]
        return df

    def get_stock_buy_trade_df(self, scode, start_date, end_date):
        """주식 매수 거래자 추출"""
        date_ranges = self.split_date_ranges(start_date, end_date, 2)
        data = self.get_stock_trader_data()
        data['isuCd'] = self.scode_lcode_dict[scode]
        data['askBid'] = '2'
        df = self.fetcher.fetch_with_date_ranges(data,'output', date_ranges)
        df.columns = [
            '날짜', '금융투자', '보험', '투신', '사모', '은행','기타금융', '연기금', '기타법인',
            '개인', '외국인','기타외국인', '전체'
            ]
        return df

    def get_stock_sell_trade_df(self, scode, start_date, end_date):
        """주식 매도 거래자 추출"""
        date_ranges = self.split_date_ranges(start_date, end_date, 2)
        data = self.get_stock_trader_data()
        data['isuCd'] = self.scode_lcode_dict[scode]
        data['askBid'] = '1'
        df = self.fetcher.fetch_with_date_ranges(data,'output', date_ranges)
        df.columns = [
            '날짜', '금융투자', '보험', '투신', '사모', '은행','기타금융', '연기금', '기타법인',
            '개인', '외국인','기타외국인', '전체'
            ]
        return df

    def get_market_trader_data(self):
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT02203',
            'locale': 'ko_KR',
            'inqTpCd': '2',
            'trdVolVal': '2',
            'detailView': '1',
            'money': '3',
            'csvxls_isNo': 'false',
        }
        return data

    def get_stock_trader_data(self):
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT02303',
            'locale': 'ko_KR',
            'inqTpCd': '2',
            'trdVolVal': '2',
            'detailView': '1',
            'money': '3',
            'csvxls_isNo': 'false',
        }
        return data



import pandas as pd
from ._data_reader import _DataReader


class EconomicIndexDataReader(_DataReader):

    def get_macro_economic_index_df(self, start_date, end_date):
        """주요경기 지표"""
        composite_leading_index = self.ecos_fetcher.fetch_item_value_df('901Y067', 'M', start_date, end_date, 'I16B')
        composite_leading_index = self.df2series(composite_leading_index, '경기종합지수')

        facility_investment_index = self.ecos_fetcher.fetch_item_value_df('901Y066', 'M', start_date, end_date, 'I15A')
        facility_investment_index = self.df2series(facility_investment_index, '설비투자지수')

        manufacturing_index = self.ecos_fetcher.fetch_item_value_df('901Y032', 'M', start_date, end_date, 'I11A')
        manufacturing_index = manufacturing_index[manufacturing_index['ITEM_CODE2'].isin(['1', '3', '5'])]
        manufacturing_index = manufacturing_index.pivot(index='TIME', columns='ITEM_NAME2', values='DATA_VALUE')
        manufacturing_index.columns.name = None

        economic_index_df = pd.concat([composite_leading_index, facility_investment_index, manufacturing_index], axis=1)
        return economic_index_df

    def get_unemployment_cnt_df(self, start_date, end_date):
        """실업급여 수"""
        unemployment_cnt_df = self.ecos_fetcher.fetch_item_value_df('901Y084', 'M', start_date, end_date)
        unemployment_cnt_df = unemployment_cnt_df[unemployment_cnt_df['ITEM_CODE2'] == 'P']
        unemployment_cnt_df = unemployment_cnt_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        unemployment_cnt_df.columns.name = None
        return unemployment_cnt_df

    def get_unemployment_won_df(self, start_date, end_date):
        """실업급여 금액"""
        unemployment_won_df = self.ecos_fetcher.fetch_item_value_df('901Y084', 'M', start_date, end_date)
        unemployment_won_df = unemployment_won_df[unemployment_won_df['ITEM_CODE2'] == 'A']
        unemployment_won_df = unemployment_won_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        unemployment_won_df.columns.name = None
        return unemployment_won_df

    def get_house_info_df(self, start_date, end_date):
        """미분양"""
        unselled_house_df = self.ecos_fetcher.fetch_item_value_df('901Y074', 'M', start_date, end_date)
        unselled_house_df = unselled_house_df[unselled_house_df['ITEM_CODE1'].isin(['I410A', 'I410B'])]
        unselled_house_df = unselled_house_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        unselled_house_df.columns.name = None
        unselled_house_df.columns = [f'미분양_{column}' for column in unselled_house_df.columns]

        approved_house_df = self.ecos_fetcher.fetch_item_value_df('901Y037', 'M', start_date, end_date)
        approved_house_df = approved_house_df[
            (approved_house_df['ITEM_CODE1'].str.startswith('I43AB')) & (approved_house_df['ITEM_CODE2'] == '1')
        ]
        approved_house_df = approved_house_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE').loc[
            :, ['용도별', '주거용', '상업용']
        ]
        approved_house_df.columns.name = None
        approved_house_df.columns = [f'건축허가_{column}' for column in approved_house_df.columns]

        building_house_df = self.ecos_fetcher.fetch_item_value_df('901Y103', 'M', start_date, end_date)
        building_house_df = building_house_df[
            (building_house_df['ITEM_CODE2'].str.startswith('I47AB')) & (building_house_df['ITEM_CODE1'] == '1')
        ]
        building_house_df = building_house_df.pivot(index='TIME', columns='ITEM_NAME2', values='DATA_VALUE').loc[
            :, ['용도별', '주거용', '상업용']
        ]
        building_house_df.columns.name = None
        building_house_df.columns = [f'건축착공_{column}' for column in building_house_df.columns]
        house_info_df = pd.concat([unselled_house_df, approved_house_df, building_house_df], axis=1)
        return house_info_df



from ..fetcher import KrxFetcher
from .base import DataReader

class ForeignDataReader(DataReader):
    def __init__(self, scode_lcode_dict):
        super().__init__()
        self.scode_lcode_dict = scode_lcode_dict
    
    def get_kospi_foreign_hold_df(self, start_date, end_date):
        """코스피 외국인 보유량"""
        date_ranges = self.split_date_ranges(start_date, end_date, 2)
        data = self.get_market_foreign_hold_data()
        data['mktId'] = 'STK'
        
        df = self.fetcher.fetch_with_date_ranges(data,'block1', date_ranges)
        df.columns = [
            '날짜','전체_시가총액','외국인_시가총액','외국인_시가총액_비율','전체_주식수','외국인_주식수','외국인_주식수_비율'
            ]
        return df

    def get_kosdaq_foreign_hold_df(self, start_date, end_date):
        """코스닥 외국인 보유량"""
        date_ranges = self.split_date_ranges(start_date, end_date, 2)
        data = self.get_market_foreign_hold_data()
        data['mktId'] = 'KSQ'
        
        df = self.fetcher.fetch_with_date_ranges(data,'block1', date_ranges)
        df.columns = [
            '날짜','전체_시가총액','외국인_시가총액','외국인_시가총액_비율','전체_주식수','외국인_주식수','외국인_주식수_비율'
            ]
        return df

    def get_stock_foreign_hold_df(self, scode, start_date, end_date):
        """주식 외국인 보유량"""
        date_ranges = self.split_date_ranges(start_date, end_date, 2)
        data = self.get_stock_foreign_hold_data()
        data['isuCd'] = self.scode_lcode_dict[scode]
        df = self.fetcher.fetch_with_date_ranges(data, 'output', date_ranges)
        df = df.drop(columns=['FLUC_TP_CD'])
        df.columns = [
            '날짜','종가','대비','등락률','전체_주식수','외국인_주식수','외국인_주식수_비율','외국인_한도수량','외국인_한도소진율'
            ]
        return df

    def get_market_foreign_hold_data(self):
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT03601',
            'locale': 'ko_KR',
            'segTpCd': 'ALL',
            'share': '2',
            'money': '3',
            'csvxls_isNo': 'false',
        }
        return data

    def get_stock_foreign_hold_data(self):
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT03702',
            'locale': 'ko_KR',
            'searchType': '2',
            'mktId': 'ALL',
            'param1isuCd_finder_stkisu0_1': 'ALL',
            'share': '1',
            'csvxls_isNo': 'false',
        }
        return data



import pandas as pd


def get_chunked_list(_list, n):
    return [_list[i : i + n] for i in range(0, len(_list), n)]


def safe_concat(df_list):
    filtered_df_list = list(filter(lambda x: x is not None, df_list))
    if filtered_df_list:
        return pd.concat(filtered_df_list, axis=0)
    return pd.DataFrame()



import pandas as pd
import FinanceDataReader as finance_data_reader
from .KrxDataReader import KrxDataReader


class MarketDataReader:
    def __init__(self):
        self.finance_data_reader = finance_data_reader
        self.krx_data_reader = KrxDataReader()

    def get_daily_info_df(self):
        daily_info_df = self.finance_data_reader.StockListing('KRX')
        rename_dict = {
            'Code': 'stock_code',
            'Name': 'stock_name',
            'Market': 'market',
            'Stocks': 'stock_shares',
            'Marcap': 'marcap',
        }
        daily_info_df = daily_info_df.rename(columns=rename_dict)
        daily_info_df = daily_info_df.loc[:, list(rename_dict.values())]
        return daily_info_df

    def get_kospi_info_df(self):
        """
        stock_name, stock_code, sector
        """
        kospi_sector_df = self.krx_data_reader.sector_data_reader.get_kospi_sector_df()
        kospi_sector_df['시가총액'] = kospi_sector_df['시가총액'].str.replace(',', '').astype(float)
        kospi_sector_df['종가'] = kospi_sector_df['종가'].str.replace(',', '').astype(float)
        kospi_sector_df['총주식수'] = (kospi_sector_df['시가총액'] / kospi_sector_df['종가']).astype(int)

        rename_dict = {
            '종목코드': 'stock_code',
            '종목명': 'stock_name',
            '업종명': 'sector',
            '총주식수': 'shares',
        }

        kospi_sector_df = kospi_sector_df.rename(columns=rename_dict)
        kospi_info_df = kospi_sector_df.loc[:, list(rename_dict.values())]
        return kospi_info_df

    def get_kospi_ohlcv_df(self, start_date, end_date):
        """
        date, open, high, low, close, volume, marcap
        """
        kospi_ohlcv_df = self.finance_data_reader.DataReader('KOSPI', start_date, end_date)

        kospi_ohlcv_df.columns = [col.lower() for col in kospi_ohlcv_df.columns]
        kospi_ohlcv_df.index.name = 'date'

        kospi_ohlcv_df = kospi_ohlcv_df.loc[:, ['open', 'high', 'low', 'close', 'volume', 'marcap']]
        return kospi_ohlcv_df

    def get_kospi_trader_df(self, start_date, end_date):
        """
        date, individual, foreign, corp
        """
        ###
        kospi_buy_trade_df = self.krx_data_reader.trader_data_reader.get_kospi_buy_trade_df(start_date, end_date)
        kospi_buy_trade_df = kospi_buy_trade_df.set_index('날짜')
        kospi_buy_trade_df = kospi_buy_trade_df.map(lambda x: x.replace(',', '')).astype(float)
        buy_trade_df = pd.concat(
            [
                kospi_buy_trade_df.loc[:, '전체'],
                kospi_buy_trade_df.loc[:, ['금융투자', '보험', '투신', '사모', '은행', '기타금융', '기타법인']]
                .sum(axis=1)
                .rename('기관'),
                kospi_buy_trade_df.loc[:, ['외국인', '기타외국인']].sum(axis=1).rename('외국인'),
                kospi_buy_trade_df.loc[:, '연기금'],
                kospi_buy_trade_df.loc[:, '개인'],
            ],
            axis=1,
        )
        buy_trade_df.columns = [col + '_매수' for col in buy_trade_df.columns]
        ###
        kospi_sell_trade_df = self.krx_data_reader.trader_data_reader.get_kospi_sell_trade_df(start_date, end_date)
        kospi_sell_trade_df = kospi_sell_trade_df.set_index('날짜')
        kospi_sell_trade_df = kospi_sell_trade_df.map(lambda x: x.replace(',', '')).astype(float)

        sell_trade_df = pd.concat(
            [
                kospi_sell_trade_df.loc[:, '전체'],
                kospi_sell_trade_df.loc[:, ['금융투자', '보험', '투신', '사모', '은행', '기타금융', '기타법인']]
                .sum(axis=1)
                .rename('기관'),
                kospi_sell_trade_df.loc[:, ['외국인', '기타외국인']].sum(axis=1).rename('외국인'),
                kospi_sell_trade_df.loc[:, '연기금'],
                kospi_sell_trade_df.loc[:, '개인'],
            ],
            axis=1,
        )
        sell_trade_df.columns = [col + '_매도' for col in sell_trade_df.columns]
        kospi_trade_df = pd.concat([buy_trade_df, sell_trade_df], axis=1)
        kospi_trade_df.index.name = 'date'

        kospi_trade_df.index = pd.to_datetime(kospi_trade_df.index, format='%Y/%m/%d')
        return kospi_trade_df

    def get_kosdaq_info_df(self):
        """
        stock_name, stock_code, sector
        """
        kosdaq_sector_df = self.krx_data_reader.sector_data_reader.get_kosdaq_sector_df()
        kosdaq_sector_df['시가총액'] = kosdaq_sector_df['시가총액'].str.replace(',', '').astype(float)
        kosdaq_sector_df['종가'] = kosdaq_sector_df['종가'].str.replace(',', '').astype(float)
        kosdaq_sector_df['총주식수'] = (kosdaq_sector_df['시가총액'] / kosdaq_sector_df['종가']).astype(int)

        rename_dict = {
            '종목코드': 'stock_code',
            '종목명': 'stock_name',
            '업종명': 'sector',
            '총주식수': 'shares',
        }

        kosdaq_sector_df = kosdaq_sector_df.rename(columns=rename_dict)
        kosdaq_info_df = kosdaq_sector_df.loc[:, list(rename_dict.values())]
        return kosdaq_info_df

    def get_kosdaq_ohlcv_df(self, start_date, end_date):
        """
        date, open, high, low, close, volume
        """
        kosdaq_ohlcv_df = self.finance_data_reader.DataReader('KOSDAQ', start_date, end_date)

        kosdaq_ohlcv_df.columns = [col.lower() for col in kosdaq_ohlcv_df.columns]
        kosdaq_ohlcv_df.index.name = 'date'

        kosdaq_ohlcv_df = kosdaq_ohlcv_df.loc[:, ['open', 'high', 'low', 'close', 'volume', 'marcap']]
        return kosdaq_ohlcv_df

    def get_kosdaq_trader_df(self, start_date, end_date):
        """
        date, individual, foreign, corp
        """
        kosdaq_buy_trade_df = self.krx_data_reader.trader_data_reader.get_kosdaq_buy_trade_df(start_date, end_date)
        kosdaq_buy_trade_df = kosdaq_buy_trade_df.set_index('날짜')
        kosdaq_buy_trade_df = kosdaq_buy_trade_df.map(lambda x: x.replace(',', '')).astype(float)
        buy_trade_df = pd.concat(
            [
                kosdaq_buy_trade_df.loc[:, '전체'],
                kosdaq_buy_trade_df.loc[:, ['금융투자', '보험', '투신', '사모', '은행', '기타금융', '기타법인']]
                .sum(axis=1)
                .rename('기관'),
                kosdaq_buy_trade_df.loc[:, ['외국인', '기타외국인']].sum(axis=1).rename('외국인'),
                kosdaq_buy_trade_df.loc[:, '연기금'],
                kosdaq_buy_trade_df.loc[:, '개인'],
            ],
            axis=1,
        )
        buy_trade_df.columns = [col + '_매수' for col in buy_trade_df.columns]
        ###
        kosdaq_sell_trade_df = self.krx_data_reader.trader_data_reader.get_kosdaq_sell_trade_df(start_date, end_date)
        kosdaq_sell_trade_df = kosdaq_sell_trade_df.set_index('날짜')
        kosdaq_sell_trade_df = kosdaq_sell_trade_df.map(lambda x: x.replace(',', '')).astype(float)

        sell_trade_df = pd.concat(
            [
                kosdaq_sell_trade_df.loc[:, '전체'],
                kosdaq_sell_trade_df.loc[:, ['금융투자', '보험', '투신', '사모', '은행', '기타금융', '기타법인']]
                .sum(axis=1)
                .rename('기관'),
                kosdaq_sell_trade_df.loc[:, ['외국인', '기타외국인']].sum(axis=1).rename('외국인'),
                kosdaq_sell_trade_df.loc[:, '연기금'],
                kosdaq_sell_trade_df.loc[:, '개인'],
            ],
            axis=1,
        )
        sell_trade_df.columns = [col + '_매도' for col in sell_trade_df.columns]
        kosdaq_trade_df = pd.concat([buy_trade_df, sell_trade_df], axis=1)
        kosdaq_trade_df.index.name = 'date'

        kosdaq_trade_df.index = pd.to_datetime(kosdaq_trade_df.index, format='%Y/%m/%d')
        return kosdaq_trade_df

    def get_stock_ohlcv_df(self, stock_code, start_date, end_date):
        """
        date, open, high, low, close, volume
        """
        stock_ohlcv_df = self.finance_data_reader.DataReader(stock_code, start_date, end_date)
        stock_ohlcv_df.columns = [col.lower() for col in stock_ohlcv_df.columns]
        stock_ohlcv_df.index.name = 'date'
        return stock_ohlcv_df

    def get_stocks_ohlcv_df(self, stock_codes, start_date, end_date):
        """
        date, open, high, low, close, volume
        """
        ohlcv_dfs = list()
        for stock_code in stock_codes:
            _ohlcv_df = self.finance_data_reader.DataReader(stock_code, start_date, end_date)
            _ohlcv_df['stock_code'] = stock_code
            ohlcv_dfs.append(_ohlcv_df)
        ohlcv_df = pd.concat(ohlcv_dfs, axis=0)
        ohlcv_df.columns = [col.lower() for col in ohlcv_df.columns]
        ohlcv_df.index.name = 'date'
        return ohlcv_df

    def get_stock_trader_df(self, stock_code, start_date, end_date):
        """
        date, individual, foreign, corp
        """
        stock_buy_trade_df = self.krx_data_reader.trader_data_reader.get_stock_buy_trade_df(stock_code, start_date, end_date)
        stock_buy_trade_df = stock_buy_trade_df.set_index('날짜')
        stock_buy_trade_df = stock_buy_trade_df.map(lambda x: x.replace(',', '')).astype(float)
        buy_trade_df = pd.concat(
            [
                stock_buy_trade_df.loc[:, '전체'],
                stock_buy_trade_df.loc[:, ['금융투자', '보험', '투신', '사모', '은행', '기타금융', '기타법인']]
                .sum(axis=1)
                .rename('기관'),
                stock_buy_trade_df.loc[:, ['외국인', '기타외국인']].sum(axis=1).rename('외국인'),
                stock_buy_trade_df.loc[:, '연기금'],
                stock_buy_trade_df.loc[:, '개인'],
            ],
            axis=1,
        )
        buy_trade_df.columns = [col + '_매수' for col in buy_trade_df.columns]
        ###
        stock_sell_trade_df = self.krx_data_reader.trader_data_reader.get_stock_sell_trade_df(stock_code, start_date, end_date)
        stock_sell_trade_df = stock_sell_trade_df.set_index('날짜')
        stock_sell_trade_df = stock_sell_trade_df.map(lambda x: x.replace(',', '')).astype(float)

        sell_trade_df = pd.concat(
            [
                stock_sell_trade_df.loc[:, '전체'],
                stock_sell_trade_df.loc[:, ['금융투자', '보험', '투신', '사모', '은행', '기타금융', '기타법인']]
                .sum(axis=1)
                .rename('기관'),
                stock_sell_trade_df.loc[:, ['외국인', '기타외국인']].sum(axis=1).rename('외국인'),
                stock_sell_trade_df.loc[:, '연기금'],
                stock_sell_trade_df.loc[:, '개인'],
            ],
            axis=1,
        )
        sell_trade_df.columns = [col + '_매도' for col in sell_trade_df.columns]
        stock_trade_df = pd.concat([buy_trade_df, sell_trade_df], axis=1)
        stock_trade_df.index.name = 'date'

        stock_trade_df.index = pd.to_datetime(stock_trade_df.index, format='%Y/%m/%d')
        return stock_trade_df



from .base import DataReader

class SectorDataReader(DataReader):
    def __init__(self):
        super().__init__()

    def get_kospi_sector_df(self):
        """코스피 sector 정보"""
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT03901',
            'locale': 'ko_KR',
            'mktId': 'STK',
            'trdDd': self.get_latest_weekday().strftime("%Y%m%d"),
            'money': '1',
            'csvxls_isNo': 'false',
        }
        df = self.fetcher.fetch(data, 'block1')
        df = df.iloc[:,:-1]
        df.columns = [
            '종목코드','종목명','시장구분','업종명','종가','대비','등락률','시가총액'
            ]
        return df

    def get_kosdaq_sector_df(self):
        """코스닥 sector 정보"""
        data = {
            'bld': 'dbms/MDC/STAT/standard/MDCSTAT03901',
            'locale': 'ko_KR',
            'mktId': 'KSQ',
            # 'segTpCd': 'ALL',
            'trdDd': self.get_latest_weekday().strftime("%Y%m%d"),
            'money': '1',
            'csvxls_isNo': 'false',
        }
        df = self.fetcher.fetch(data, 'block1')
        df = df.iloc[:,:-1]
        df.columns = [
            '종목코드','종목명','시장구분','업종명','종가','대비','등락률','시가총액'
            ]
        return df



import pandas as pd
from ._data_reader import _DataReader


class BankDataReader(_DataReader):
    def get_deposit_df(self, start_date, end_date):
        """예금 말잔"""
        deposit_df = self.ecos_fetcher.fetch_item_value_df('104Y013', 'M', start_date, end_date)
        deposit_df = deposit_df[deposit_df['ITEM_CODE1'].isin(['BCB1', 'BCB2', 'BCB8'])]
        deposit_df = deposit_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        deposit_df.columns.name = None
        return deposit_df

    def get_deposit_by_owner_df(self, start_date, end_date):
        """예금 집단별"""
        deposit_by_owner_df = self.ecos_fetcher.fetch_item_value_df('104Y009', 'M', start_date, end_date)
        deposit_by_owner_df = deposit_by_owner_df[
            deposit_by_owner_df['ITEM_CODE1'].isin(['1000000', '1010000', '1020000', '1030000', '1040000', '1050000'])
        ]
        deposit_by_owner_df = deposit_by_owner_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        deposit_by_owner_df.columns.name = None
        return deposit_by_owner_df

    def get_loan_df(self, start_date, end_date):
        """부채 말잔"""
        loan_df = self.ecos_fetcher.fetch_item_value_df('104Y016', 'M', start_date, end_date)
        loan_df = loan_df[loan_df['ITEM_CODE1'].isin(['BDCA21', 'BDCA31', 'BDCA2E', 'BDCA1'])]
        loan_df = loan_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        loan_df.columns.name = None
        return loan_df

    def get_overdue_loan_rate_df(self, start_date, end_date):
        """연체율"""
        overdue_loan_rate_df = self.ecos_fetcher.fetch_item_value_df('901Y054', 'M', start_date, end_date)
        ('MO3AB', 'AB'), ('MO3AA', 'AB'), ('MO3AC', 'AB')
        overdue_loan_rate_df = overdue_loan_rate_df[
            ((overdue_loan_rate_df['ITEM_CODE1'] == 'MO3AB') & (overdue_loan_rate_df['ITEM_CODE2'] == 'AB'))
            | ((overdue_loan_rate_df['ITEM_CODE1'] == 'MO3AA') & (overdue_loan_rate_df['ITEM_CODE2'] == 'AB'))
            | ((overdue_loan_rate_df['ITEM_CODE1'] == 'MO3AC') & (overdue_loan_rate_df['ITEM_CODE2'] == 'AB'))
        ]
        overdue_loan_rate_df = overdue_loan_rate_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        overdue_loan_rate_df.columns.name = None
        return overdue_loan_rate_df

    def get_bok_asset_df(self, start_date, end_date):
        """한국은행 자산"""
        bok_assets_df = self.ecos_fetcher.fetch_item_value_df('732Y001', 'M', start_date, end_date)
        bok_assets_df = bok_assets_df.pivot(index='TIME', columns='ITEM_NAME1', values='DATA_VALUE')
        bok_assets_df.columns.name = None
        return bok_assets_df



import pandas as pd
from ._data_reader import _DataReader


class InterestDataReader(_DataReader):
    """금리"""

    def get_interest_rate_df(self, start_date, end_date):
        """금리"""
        # 한국은행 기준금리
        base_ir = self.ecos_fetcher.fetch_item_value_df('722Y001', 'M', start_date, end_date, '0101000')
        base_ir = self.df2series(base_ir, '기준금리')

        # 수신금리
        deposit_ir = self.ecos_fetcher.fetch_item_value_df('121Y002', 'M', start_date, end_date, 'BEABAA2')
        deposit_ir = self.df2series(deposit_ir, '수신금리')

        # 대출금리
        loan_ir = self.ecos_fetcher.fetch_item_value_df('121Y006', 'M', start_date, end_date, 'BECBLA01')
        loan_ir = self.df2series(loan_ir, '대출금리')
        interest_rate_df = pd.concat([base_ir, deposit_ir, loan_ir], axis=1)
        return interest_rate_df

    def get_national_treasury_df(self, start_date, end_date):
        """국고채"""
        # 국고채 1년
        nt_1_year = self.ecos_fetcher.fetch_item_value_df('817Y002', 'D', start_date, end_date, '010190000')
        nt_1_year = self.df2series(nt_1_year, '국고채_1년')

        # 국고채 3년
        nt_3_year = self.ecos_fetcher.fetch_item_value_df('817Y002', 'D', start_date, end_date, '010200000')
        nt_3_year = self.df2series(nt_3_year, '국고채_3년')

        # 국고채 5년
        nt_5_year = self.ecos_fetcher.fetch_item_value_df('817Y002', 'D', start_date, end_date, '010200001')
        nt_5_year = self.df2series(nt_5_year, '국고채_5년')

        # 국고채 10년
        nt_10_year = self.ecos_fetcher.fetch_item_value_df('817Y002', 'D', start_date, end_date, '010210000')
        nt_10_year = self.df2series(nt_10_year, '국고채_10년')

        # 국고채 20년
        nt_20_year = self.ecos_fetcher.fetch_item_value_df('817Y002', 'D', start_date, end_date, '010220000')
        nt_20_year = self.df2series(nt_20_year, '국고채_20년')

        national_treasury_df = pd.concat([nt_1_year, nt_3_year, nt_5_year, nt_10_year, nt_20_year], axis=1)
        return national_treasury_df



from setuptools import setup, find_packages

setup(
    name="KFinanceDataReader",  # 패키지 이름 (PyPI에 표시될 이름)
    version="1.0.5",  # 패키지 버전
    author="jackmappotion",  # 작성자 이름
    author_email="jackmappotion@gmail.com",  # 작성자 이메일
    description="한국 주요 금융 데이터 추출을 위한 라이브러리",  # 패키지 설명
    long_description=open("README.md", "r", encoding="utf-8").read(),  # 상세 설명 (README.md를 읽어 사용)
    long_description_content_type="text/markdown",  # README 파일 형식 (주로 markdown 사용)
    url="https://github.com/jackmappotion/KFinanceDataReader",  # 프로젝트 GitHub URL
    packages=find_packages(),  # 패키지 자동 탐색
    classifiers=[  # PyPI 분류 정보
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",  # 지원 Python 버전
    install_requires=[  # 의존성 패키지
        "requests>=2.0.0",
        "pandas>=2.0.0",
        "finance-datareader",
        "plotly",
        "xmltodict",
    ],
)



from ._macro_data_reader import MacroDataReader
from ._market_data_reader import MarketDataReader
from ._corp_data_reader import CorpDataReader


class KFinanceDataReader:
    def __init__(self):
        self.MacroDataReader = MacroDataReader
        self.MarketDataReader = MarketDataReader
        self.CorpDataReader = CorpDataReader



import pandas as pd
from ..fetcher import EcosFetcher


class _DataReader:
    def __init__(self, api_key):
        self.api_key = api_key
        self.ecos_fetcher = EcosFetcher(api_key)

    @staticmethod
    def df2series(df, name):
        series = df.set_index("TIME")['DATA_VALUE']
        series.name = name
        return series
